﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Termination {
  [Item("ComparisonTerminator", "An termination criterion which compares two values.")]
  [StorableClass]
  public class ComparisonTerminator<T> : ThresholdTerminator<T> where T : class, IItem, IComparable {
    public ILookupParameter<T> ValueParameter {
      get { return (ILookupParameter<T>)Parameters["Value"]; }
    }

    private ValueParameter<Comparison> ComparisonParameter {
      get { return (ValueParameter<Comparison>)Parameters["Comparison"]; }
    }

    public Comparison Comparison {
      get { return ComparisonParameter.Value; }
      set { ComparisonParameter.Value = value; }
    }

    [StorableConstructor]
    protected ComparisonTerminator(bool deserializing) : base(deserializing) { }
    protected ComparisonTerminator(ComparisonTerminator<T> original, Cloner cloner)
      : base(original, cloner) { }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new ComparisonTerminator<T>(this, cloner);
    }
    public ComparisonTerminator()
      : base() {
      Parameters.Add(new LookupParameter<T>("Value", "The left side value of the comparison."));
      Parameters.Add(new ValueParameter<Comparison>("Comparison", "The type of comparison.", new Comparison(ComparisonType.GreaterOrEqual)) { Hidden = true });
    }
    public ComparisonTerminator(string leftSideActualName, string rightSideActualName, ComparisonType comparison = ComparisonType.GreaterOrEqual)
      : this() {
      ValueParameter.ActualName = leftSideActualName;
      ThresholdParameter.ActualName = rightSideActualName;
      Comparison.Value = comparison;
    }
    public ComparisonTerminator(string leftSideActualName, T rightSideValue, ComparisonType comparison = ComparisonType.GreaterOrEqual)
      : this() {
      ValueParameter.ActualName = leftSideActualName;
      ThresholdParameter.Value = rightSideValue;
      Comparison.Value = comparison;
    }

    protected override bool CheckTermination() {
      IComparable lhs = ValueParameter.ActualValue;
      IComparable rhs = ThresholdParameter.ActualValue;

      int i = lhs.CompareTo(rhs);
      switch (Comparison.Value) {
        case ComparisonType.Less:
          return i < 0;
        case ComparisonType.LessOrEqual:
          return i <= 0;
        case ComparisonType.Equal:
          return i == 0;
        case ComparisonType.GreaterOrEqual:
          return i >= 0;
        case ComparisonType.Greater:
          return i > 0;
        case ComparisonType.NotEqual:
          return i != 0;
        default:
          throw new InvalidOperationException(Name + ": " + Comparison.Value + " is not supported.");
      }
    }
  }
}